<?php
namespace Fiscal\Logic\NotaFiscal\Saida;

use Financeiro\Entities\Contareceber;
use Fiscal\Entities\Notafiscal;
use Fiscal\Logic\NotaFiscal\ConferenciaErros;
use Fiscal\Logic\NotaFiscal\Exceptions;
use Fiscal\Logic\NotaFiscal\Lancamento\IEvents;
use Fiscal\Logic\NotaFiscal\ProceduresNF;
use Fiscal\Util\Util;
use Geral\Entities\Itemencargo;
use Geral\Entities\Itemnotafisc;
use Geral\Entities\Itemproduto;
use Geral\Entities\Tipooperac;
use Geral\Logic\Produto\Produto;
use Geral\Util\Ambiente;
use Geral\Util\DateUtil;
use Setforms2\Shared\Registry;

class NFSaida implements IEvents
{

    /**@var $rateioAnterior array */
    private $rateioAnterior;

    /**@var $notafiscal \Fiscal\Entities\Notafiscal */
    private $notafiscal;

    /**@var $totalNF float */
    private $totalNF;

    /**
     * Guarda instancia da notafiscal e inicia instancia do validador de NF de saída.
     * @param \Fiscal\Entities\Notafiscal $notafiscal
     */
    public function __construct(Notafiscal $notafiscal)
    {
        $this->setNotaFiscal($notafiscal);
        $this->validator = new NFValida($this->notafiscal);
    }

    /**
     * Gera o novo codigo da nota fiscal.
     *
     * @static
     * @return number
     *
     */
    public static function novoCodigo()
    {
        $cdfilial = \Geral\Util\Ambiente::getCdFilial();
        $nrorg    = \Geral\Util\Ambiente::getNrorg();
        return \Geral\Util\NovoCodigo::geraCodigo($nrorg, 'LANCNF/FILIAL' . $cdfilial);
    }

    public static function deleteProdutosNF(Itemproduto $produto, $lancamento, $nrseqitprod)
    {
        $itemProduto = Itemproduto::findOneByNrlanctonfAndNrseqitprodAndCdfilialAndNrorg($lancamento, $nrseqitprod, $produto->getCdfilial(), $produto->getNrorg());
        $itemProduto->delete();
        Registry::getEntityManager()->flush();
    }

    public static function saveEncargosNFDevolucao($nrorg, $filial, $nrlanctonf, $encargos)
    {
        foreach ($encargos as $encargo) {
            $manager = Registry::getEntityManager();

            $itemencargo = new Itemencargo();

            $itemencargo->setCdfilial($filial);
            $itemencargo->setNrlanctonf($nrlanctonf);
            $itemencargo->setCdencargo($encargo->getCdencargo());
            $itemencargo->setVritemenca($encargo->getVritemenca());
            $itemencargo->setNrorg($nrorg);
            $itemencargo->setDtinclusao(DateUtil::getDataAtual());
            $itemencargo->setDtultatu(DateUtil::getDataAtual());
            $itemencargo->setCdoperinclusao(Ambiente::getCdOperador());
            $itemencargo->setCdoperultatu(Ambiente::getCdOperador());

            $itemencargo->persist();

            $manager->flush();
        }
    }

    public static function salvaRelacaoNotafiscalOrigem($lancamento, $lancamentoOrigem, $filialOrigem)
    {
        $itemnotafisc = new Itemnotafisc();

        $itemnotafisc->setCdfilial(Ambiente::getCdFilial());
        $itemnotafisc->setNrlanctonf($lancamento);
        $itemnotafisc->setNrsequitnota('001');
        $itemnotafisc->setCdfilialit($filialOrigem);
        $itemnotafisc->setNrlanctonfit($lancamentoOrigem);
        $itemnotafisc->setNrorg(Ambiente::getNrorg());
        $itemnotafisc->setDtinclusao(DateUtil::getDataAtual());
        $itemnotafisc->setDtultatu(DateUtil::getDataAtual());
        $itemnotafisc->setNrorginclusao(Ambiente::getNrorg());
        $itemnotafisc->setCdoperinclusao(Ambiente::getCdOperador());
        $itemnotafisc->setNrorgultatu(Ambiente::getNrorg());
        $itemnotafisc->setCdoperultatu(Ambiente::getCdOperador());

        $itemnotafisc->persist();

        Registry::getEntityManager()->flush();
    }

    public static function getProximoNRSEQVENREC($filial, $lancamento)
    {
        $nrseqvenrec = Contareceber::getRepository()->getMaxNRSEQVENREC($filial, $lancamento) + 1;

        return str_pad($nrseqvenrec, 2, '0', STR_PAD_LEFT);
    }

    public static function atualizaQtrelacNforigem($cdfilial, $nrlanctonf, $nrseqitprod, $vrdiferenca, $idstatus)
    {
        ProceduresNF::callProcedureAtualizaQtrelacNforigem($cdfilial, $nrlanctonf, $nrseqitprod, $vrdiferenca, $idstatus);
    }

    /**
     * @param $tipoOperacao
     * @param $update
     * @param $itemproduto
     */
    public static function informacoesComplementaresProduto(Tipooperac $tipoOperacao, $update, Itemproduto $itemproduto)
    {
        $boDocuDevo = false;
        if ($tipoOperacao->getIdoperdevonf() == 'S') {
            $boDocuDevo = true;
        }

        if ($tipoOperacao->getCdtipooper() != '03') {
            if ($update) {
                Produto::openInfComplProd(
                    $itemproduto->getCdfilial(), DateUtil::getDataAtual(),
                    $itemproduto->getCdproduto(), $itemproduto->getCdalmoxarife(), $itemproduto->getCdlocalestoq(),
                    $itemproduto->getNrloteestq(), $itemproduto->getNrsublote(), $itemproduto->getNrorg(), false,
                    true, false, false, true, false, $boDocuDevo, false
                );
            } else {
                Produto::openInfComplProd(
                    $itemproduto->getCdfilial(), DateUtil::getDataAtual(),
                    $itemproduto->getCdproduto(), $itemproduto->getCdalmoxarife(), $itemproduto->getCdlocalestoq(),
                    $itemproduto->getNrloteestq(), $itemproduto->getNrsublote(), $itemproduto->getNrorg(), false,
                    true, false, false, true, true, $boDocuDevo, false
                );
            }
        } else {
            if ($update) {
                Produto::openInfComplProd(
                    $itemproduto->getCdfilial(), DateUtil::getDataAtual(),
                    $itemproduto->getCdproduto(), $itemproduto->getCdalmoxarife(), $itemproduto->getCdlocalestoq(),
                    $itemproduto->getNrloteestq(), $itemproduto->getNrsublote(), $itemproduto->getNrorg(), false,
                    true, false, true, true, false, $boDocuDevo, false
                );
            } else {
                Produto::openInfComplProd(
                    $itemproduto->getCdfilial(), DateUtil::getDataAtual(),
                    $itemproduto->getCdproduto(), $itemproduto->getCdalmoxarife(), $itemproduto->getCdlocalestoq(),
                    $itemproduto->getNrloteestq(), $itemproduto->getNrsublote(), $itemproduto->getNrorg(), false,
                    true, false, true, true, true, $boDocuDevo, false
                );
            }
        }
    }

    /**
     * @return \Fiscal\Entities\Notafiscal
     */
    public function getNotaFiscal()
    {
        return $this->notafiscal;
    }

    /**
     * Guarda instancia da entidade notafiscal.
     * @param \Fiscal\Entities\Notafiscal $notafiscal
     */
    protected function setNotaFiscal(Notafiscal $notafiscal)
    {
        $this->notafiscal = $notafiscal;
    }

    /**
     * Retorna total da nota fiscal
     * @return float|null
     */
    public function getTotalNF()
    {
        return $this->totalNF;
    }

    /**
     * @param $totalNF float;
     * @throws \Fiscal\Logic\NotaFiscal\Exceptions\NFValidaSaidaException
     */
    public function setTotalNF($totalNF)
    {
        if ($totalNF < 0) {
            throw new Exceptions\NFValidaSaidaException("Operação Bloqueada! Soma dos encargos com tratamento descontar superior ao valor total da nota.(Diferença: $totalNF)");
        }

        $this->totalNF = (float)$totalNF;
    }

    /**
     * Evento chamado antes do lançamento da notafiscal.
     * @throws \Exception
     */
    public function beforeLancamento()
    {
        try {
            //@TODO - DESCOMENTAR
            $this->validator->validaEmissao();
            $this->rateioAnterior = Util::geraRateioAnterior($this->notafiscal);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Evento chamado após o lançamento da notafiscal.
     * @throws \Exception
     */
    public function afterLancamento()
    {
        try {
            // Verifica e valida vencimentos da nota (caso tenha esteje com o parametro de gerar vencimentos automaticos, então GERA os mesmos).
            $this->verificaVencimentos();
            // Conferencia de erros da NF.
            ConferenciaErros::executaProcedureConferenciaDeErros($this->notafiscal);
            ConferenciaErros::confereErros($this->notafiscal);
            Util::replicacaoAutomatica($this->notafiscal);
            Util::atualizaEstoque($this->notafiscal);
            //@TODO Ajustar geracao do rateio
            Util::geraRateioPadrao($this->notafiscal);
            //            LancamentoNFSaida::atualizaStatusNF($this->notafiscal, \Fiscal\Entities\Notafiscal::PENDENTE);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Gera vencimentos caso esteja habilitado "vencimento automático"
     * Valida vencimentos.
     * Valida total NF com vencimentos.
     * @throws \Exception
     */
    public function verificaVencimentos()
    {
        try {
            $gerador = new \Fiscal\Util\GeradorVencimentos($this->notafiscal);
            if ($gerador->utilizaVencimento()) {
                if ($gerador->permiteGeracaoAutomatica()) {
                    $contareceber = Contareceber::findOneByNrorgAndCdfilialAndNrlanctonf($this->notafiscal->getNrorg(), $this->notafiscal->getCdfilial(), $this->notafiscal->getNrlanctonf());
                    if (!$contareceber instanceof Contareceber) {
                        $gerador->gerarVencimentos();
                    }
                }

                if ($gerador->utilizaVencimentoCobranca()) {
                    $this->validator->validaVencimentosCobranca($this->notafiscal);
                }
            }
            // 			else{
            // 				$this->validator->validaTotalNF($this->totalNF);
            // 			}
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveProdutosNF(Itemproduto $produto, $lancamento, $nrseqitprod = null)
    {
        if (!$nrseqitprod) {
            $itemProduto = new Itemproduto();

            $itemProduto->setNrorg($produto->getNrorg());
            $itemProduto->setDtinclusao(DateUtil::getDataAtual());
            $itemProduto->setNrorginclusao($produto->getNrorg());
            $itemProduto->setCdoperinclusao(Ambiente::getCdOperador());
            $itemProduto->setNrlanctonf($lancamento);
            $itemProduto->setCdoperador(Ambiente::getCdOperador());
            $itemProduto->setCdalmoxarife($produto->getCdalmoxarife());
            $itemProduto->setCdlocalestoq($produto->getCdlocalestoq());
            $itemProduto->setNrloteestq($produto->getNrloteestq());
            $itemProduto->setNrsublote($produto->getNrsublote());
            $itemProduto->setCdfilialped($produto->getCdfilialped());
            $itemProduto->setNrpedido($produto->getNrpedido());
            $itemProduto->setNritemped($produto->getNritemped());
            $itemProduto->setQtvoluitem($produto->getQtvoluitem());
            $itemProduto->setCdservfatu($produto->getCdservfatu());
            $itemProduto->setNrpedprod($produto->getNrpedprod());
            $itemProduto->setNrseqpedprod($produto->getNrseqpedprod());
            $itemProduto->setCdfiscopersn($produto->getCdfiscopersn());
            $itemProduto->setCdservfili($produto->getCdservfili());
            $itemProduto->setQtitrelacnfsubs($produto->getQtitrelacnfsubs());
            $itemProduto->setNrseqitprod(self::geraProximoNRSEQITPROD($produto->getCdfilial(), $lancamento));
            $itemProduto->setIdatuvrestoq($produto->getIdatuvrestoq());
            $itemProduto->setVrcontitem($produto->getVrcontitem());
            $itemProduto->setIdconforitem($produto->getIdconforitem());
            $itemProduto->setVrbrutitem($produto->getVrbrutitem());
            $itemProduto->setVrdesperitem($produto->getVrdesperitem());
            $itemProduto->setCdfilial($produto->getCdfilial());
            $itemProduto->setCdfilimovi($produto->getCdfilimovi());
            $itemProduto->setCdproduto($produto->getCdproduto());
            $itemProduto->setVrunititem($produto->getVrunititem());
            $itemProduto->setVrdescitem($produto->getVrdescitem());
            $itemProduto->setCdprodemis($produto->getCdprodemis());
            $itemProduto->setDsprodemis($produto->getDsprodemis());
            $itemProduto->setCdfisopeitem($produto->getCdfisopeitem());
            $itemProduto->setCdsufxcfoppr($produto->getCdsufxcfoppr());
            $itemProduto->setCdsitutribpr($produto->getCdsitutribpr());
            $itemProduto->setIdavisositem($produto->getIdavisositem());
        } else {
            $itemProduto = Itemproduto::findOneByNrlanctonfAndNrseqitprodAndCdfilialAndNrorg($lancamento, $nrseqitprod, $produto->getCdfilial(), $produto->getNrorg());
        }


        $itemProduto->setDtultatu(DateUtil::getDataAtual());
        $itemProduto->setNrorgultatu($produto->getNrorg());
        $itemProduto->setCdoperultatu(Ambiente::getCdOperador());
        $itemProduto->setVrtotaitem($produto->getVrtotaitem());
        $itemProduto->setQtitemprod($produto->getQtitemprod());

        $itemProduto->persist();

        Registry::getEntityManager()->flush();

        return $itemProduto->getNrseqitprod();
    }

    public static function geraProximoNRSEQITPROD($filial, $lancamento)
    {
        $nrseqitprod = Itemproduto::getRepository()->getMaxNRSEQITPROD($filial, $lancamento) + 1;

        return str_pad($nrseqitprod, 4, '0', STR_PAD_LEFT);
    }
}
